<?php

if ( ! defined( 'CHESHALO_THEME_DIR' ) ) {
	exit( 'No direct script access allowed' );
}

if ( ! function_exists( 'cheshalo_get_vc_map_text_block' ) ) {
	function cheshalo_get_vc_map_text_block() {
		$secondary_font = cheshalo_get_opt( 'secondary-font' );
		$primary_font   = cheshalo_get_opt( 'primary-font' );

		$secondary_font_title = isset( $secondary_font[0] ) ? esc_html__( 'Secondary font', 'cheshalo' ) . ' (' . $secondary_font[0]['font-family'] . ')' : esc_html__( 'Secondary font', 'cheshalo' );
		$primary_font_title   = isset( $primary_font[0] ) ? esc_html__( 'Title font', 'cheshalo' ) . ' (' . $primary_font[0]['font-family'] . ')' : esc_html__( 'Title font', 'cheshalo' );

		return array(
			'name'        => esc_html__( 'Text block', 'cheshalo' ),
			'base'        => 'cheshalo_text_block',
			'category'    => cheshalo_get_tab_title_category_for_wpb( esc_html__( 'Theme elements', 'cheshalo' ) ),
			'description' => esc_html__( 'A block of text', 'cheshalo' ),
			'icon'        => CHESHALO_ASSETS . '/images/vc-icon/text-block.svg',
			'params'      => array(
				array(
					'type'       => 'cheshalo_css_id',
					'param_name' => 'cheshalo_css_id',
				),
				/**
				 * Content.
				 */
				array(
					'type'       => 'cheshalo_title_divider',
					'holder'     => 'div',
					'title'      => esc_html__( 'Content', 'cheshalo' ),
					'param_name' => 'content_divider',
				),
				array(
					'type'       => 'textarea_html',
					'holder'     => 'div',
					'heading'    => esc_html__( 'Text', 'cheshalo' ),
					'param_name' => 'content',
					'std'        => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
				),
				array(
					'type'             => 'cheshalo_button_set',
					'heading'          => esc_html__( 'Color Scheme', 'cheshalo' ),
					'param_name'       => 'text_color_scheme',
					'value'            => array(
						esc_html__( 'Inherit', 'cheshalo' ) => 'inherit',
						esc_html__( 'Light', 'cheshalo' ) => 'light',
						esc_html__( 'Dark', 'cheshalo' ) => 'dark',
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				/**
				 * Paragraph.
				 */
				array(
					'type'       => 'cheshalo_title_divider',
					'holder'     => 'div',
					'title'      => esc_html__( 'Paragraph', 'cheshalo' ),
					'param_name' => 'paragraph_divider',
				),
				array(
					'type'             => 'dropdown',
					'heading'          => esc_html__( 'Text font', 'cheshalo' ),
					'param_name'       => 'text_font_family',
					'value'            => array(
						esc_html__( 'Default', 'cheshalo' ) => 'default',
						$primary_font_title   => 'primary',
						$secondary_font_title => 'alt',
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'             => 'dropdown',
					'heading'          => esc_html__( 'Font size', 'cheshalo' ),
					'param_name'       => 'text_font_size',
					'value'            => array(
						esc_html__( 'Default', 'cheshalo' ) => 'default',
						esc_html__( 'Extra Small (14px)', 'cheshalo' ) => 'xs',
						esc_html__( 'Small (16px)', 'cheshalo' ) => 's',
						esc_html__( 'Medium (18px)', 'cheshalo' ) => 'm',
						esc_html__( 'Large (22px)', 'cheshalo' ) => 'l',
						esc_html__( 'Extra Large (26px)', 'cheshalo' ) => 'xl',
						esc_html__( 'XXL (36px)', 'cheshalo' ) => 'xxl',
						esc_html__( 'XXXL (46px)', 'cheshalo' ) => 'xxxl',
						esc_html__( 'Custom', 'cheshalo' ) => 'custom',
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'             => 'cheshalo_responsive_size',
					'heading'          => esc_html__( 'Size', 'cheshalo' ),
					'param_name'       => 'text_font_size_custom',
					'css_args'         => array(
						'font-size' => array(
							'.wd-text-block',
						),
					),
					'dependency'       => array(
						'element' => 'text_font_size',
						'value'   => array( 'custom' ),
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'             => 'cheshalo_responsive_size',
					'heading'          => esc_html__( 'Line height', 'cheshalo' ),
					'param_name'       => 'text_line_height_custom',
					'css_args'         => array(
						'line-height' => array(
							'.wd-text-block',
						),
					),
					'dependency'       => array(
						'element' => 'text_font_size',
						'value'   => array( 'custom' ),
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'       => 'cheshalo_empty_space',
					'param_name' => 'cheshalo_empty_space',
				),
				array(
					'type'             => 'dropdown',
					'heading'          => esc_html__( 'Font weight', 'cheshalo' ),
					'param_name'       => 'text_font_weight',
					'value'            => array(
						esc_html__( 'Default', 'cheshalo' ) => 'default',
						esc_html__( 'Ultra-Light 100', 'cheshalo' ) => 100,
						esc_html__( 'Light 200', 'cheshalo' ) => 200,
						esc_html__( 'Book 300', 'cheshalo' ) => 300,
						esc_html__( 'Normal 400', 'cheshalo' ) => 400,
						esc_html__( 'Medium 500', 'cheshalo' ) => 500,
						esc_html__( 'Semi-Bold 600', 'cheshalo' ) => 600,
						esc_html__( 'Bold 700', 'cheshalo' ) => 700,
						esc_html__( 'Extra-Bold 800', 'cheshalo' ) => 800,
						esc_html__( 'Ultra-Bold 900', 'cheshalo' ) => 900,
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'             => 'cheshalo_dropdown',
					'heading'          => esc_html__( 'Color', 'cheshalo' ),
					'param_name'       => 'text_color',
					'value'            => array(
						esc_html__( 'Default', 'cheshalo' ) => 'default',
						esc_html__( 'Title', 'cheshalo' ) => 'title',
						esc_html__( 'Primary', 'cheshalo' ) => 'primary',
						esc_html__( 'Alternative', 'cheshalo' ) => 'alt',
						esc_html__( 'Custom', 'cheshalo' ) => 'custom',
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'       => 'cheshalo_colorpicker',
					'heading'    => esc_html__( 'Custom Color', 'cheshalo' ),
					'param_name' => 'text_color_custom',
					'css_args'   => array(
						'color' => array(
							'.wd-text-block',
						),
					),
					'dependency' => array(
						'element' => 'text_color',
						'value'   => array( 'custom' ),
					),
				),
				/**
				 * Layout
				 */
				array(
					'type'       => 'cheshalo_title_divider',
					'holder'     => 'div',
					'title'      => esc_html__( 'Layout', 'cheshalo' ),
					'param_name' => 'layout_divider',
				),
				array(
					'type'             => 'cheshalo_image_select',
					'heading'          => esc_html__( 'Text align', 'cheshalo' ),
					'param_name'       => 'text_align',
					'value'            => array(
						esc_html__( 'Left', 'cheshalo' )   => 'left',
						esc_html__( 'Center', 'cheshalo' ) => 'center',
						esc_html__( 'Right', 'cheshalo' )  => 'right',
					),
					'images_value'     => array(
						'center' => CHESHALO_ASSETS_IMAGES . '/settings/align/center.jpg',
						'left'   => CHESHALO_ASSETS_IMAGES . '/settings/align/left.jpg',
						'right'  => CHESHALO_ASSETS_IMAGES . '/settings/align/right.jpg',
					),
					'wood_tooltip'     => true,
					'edit_field_class' => 'vc_col-sm-6 vc_column title-align',
				),
				array(
					'type'             => 'dropdown',
					'heading'          => esc_html__( 'Content width', 'cheshalo' ),
					'param_name'       => 'content_width',
					'value'            => array(
						'100%' => '100',
						'90%'  => '90',
						'80%'  => '80',
						'70%'  => '70',
						'60%'  => '60',
						'50%'  => '50',
						'40%'  => '40',
						'30%'  => '30',
						'20%'  => '20',
						'10%'  => '10',
						esc_html__( 'Custom', 'cheshalo' ) => 'custom',
					),
					'edit_field_class' => 'vc_col-sm-6 vc_column',
				),
				array(
					'type'             => 'cheshalo_button_set',
					'heading'          => esc_html__( 'Custom content width', 'cheshalo' ),
					'param_name'       => 'custom_content_width',
					'tabs'             => true,
					'value'            => array(
						esc_html__( 'Desktop', 'cheshalo' ) => 'desktop',
						esc_html__( 'Tablet', 'cheshalo' ) => 'tablet',
						esc_html__( 'Mobile', 'cheshalo' ) => 'mobile',
					),
					'default'          => 'desktop',
					'edit_field_class' => 'wd-res-control wd-custom-width vc_col-sm-12 vc_column',
					'dependency'       => array(
						'element' => 'content_width',
						'value'   => array( 'custom' ),
					),
				),
				array(
					'type'             => 'cheshalo_slider',
					'param_name'       => 'content_desktop_width',
					'min'              => '0',
					'max'              => '1000',
					'step'             => '1',
					'default'          => '600',
					'units'            => 'px',
					'edit_field_class' => 'wd-res-item vc_col-sm-12 vc_column',
					'css_args'         => array(
						'--wd-max-width' => array(
							'',
						),
					),
					'css_params'       => array(
						'device' => 'desktop',
					),
					'wd_dependency'    => array(
						'element' => 'custom_content_width',
						'value'   => array( 'desktop' ),
					),
					'dependency'       => array(
						'element' => 'content_width',
						'value'   => array( 'custom' ),
					),
				),
				array(
					'type'             => 'cheshalo_slider',
					'param_name'       => 'content_tablet_width',
					'min'              => '0',
					'max'              => '1000',
					'step'             => '1',
					'default'          => '0',
					'units'            => 'px',
					'edit_field_class' => 'wd-res-item vc_col-sm-12 vc_column',
					'css_args'         => array(
						'--wd-max-width' => array(
							'',
						),
					),
					'css_params'       => array(
						'device' => 'tablet',
					),
					'wd_dependency'    => array(
						'element' => 'custom_content_width',
						'value'   => array( 'tablet' ),
					),
					'dependency'       => array(
						'element' => 'content_width',
						'value'   => array( 'custom' ),
					),
				),
				array(
					'type'             => 'cheshalo_slider',
					'param_name'       => 'content_mobile_width',
					'min'              => '0',
					'max'              => '1000',
					'step'             => '1',
					'default'          => '0',
					'units'            => 'px',
					'edit_field_class' => 'wd-res-item vc_col-sm-12 vc_column',
					'css_args'         => array(
						'--wd-max-width' => array(
							'',
						),
					),
					'css_params'       => array(
						'device' => 'mobile',
					),
					'wd_dependency'    => array(
						'element' => 'custom_content_width',
						'value'   => array( 'mobile' ),
					),
					'dependency'       => array(
						'element' => 'content_width',
						'value'   => array( 'custom' ),
					),
				),
				cheshalo_get_vc_display_inline_map(),
				/**
				 * Extra
				 */
				array(
					'type'       => 'cheshalo_title_divider',
					'holder'     => 'div',
					'title'      => esc_html__( 'Extra options', 'cheshalo' ),
					'param_name' => 'extra_divider',
				),
				array(
					'type'       => 'css_editor',
					'heading'    => esc_html__( 'CSS box', 'cheshalo' ),
					'param_name' => 'css',
					'group'      => esc_html__( 'Design Options', 'js_composer' ),
				),
				cheshalo_get_vc_responsive_spacing_map(),

				cheshalo_parallax_scroll_map( 'parallax_scroll' ),
				cheshalo_parallax_scroll_map( 'scroll_x' ),
				cheshalo_parallax_scroll_map( 'scroll_y' ),
				cheshalo_parallax_scroll_map( 'scroll_z' ),
				cheshalo_parallax_scroll_map( 'scroll_smooth' ),

				cheshalo_get_vc_animation_map( 'wd_animation' ),
				cheshalo_get_vc_animation_map( 'wd_animation_delay' ),
				cheshalo_get_vc_animation_map( 'wd_animation_duration' ),

				array(
					'type'       => 'textfield',
					'heading'    => esc_html__( 'Extra class name', 'cheshalo' ),
					'param_name' => 'extra_classes',
					'hint'       => esc_html__( 'If you wish to style particular content element differently, then use this field to add a class name and then refer to it in your css file.', 'cheshalo' ),
				),

				cheshalo_get_vc_responsive_visible_map( 'responsive_tabs_hide' ),
				cheshalo_get_vc_responsive_visible_map( 'wd_hide_on_desktop' ),
				cheshalo_get_vc_responsive_visible_map( 'wd_hide_on_tablet' ),
				cheshalo_get_vc_responsive_visible_map( 'wd_hide_on_mobile' ),
			),
		);
	}
}
